@@ -44,13 +44,14 @@ lPriors2 <- list(loc.mu=peakLocations, loc.sd=rep(50,nPK), scaG.mu=log(16.47) -
                  noise.sd=50, bl.smooth=1, bl.knots=50)
 data("result2", package = "serrsBayes")
 if(!exists("result2")) {
-  tm2 <- system.time(result2 <- fitVoigtPeaksSMC(wavenumbers, spectra, lPriors2, npart=3000))
-  result2$time <- tm2
+  tm2 <- Sys.time()
+  result2 <- fitVoigtPeaksSMC(wavenumbers, spectra, lPriors2, npart=3000, rate=0.499,mcSteps=50)
+  result2$time <- Sys.time() - tm2
   save(result2, file="Figure 2/result2.rda")
 }
 
 ## ----fitVoigtBL, fig.width=6--------------------------------------------------
-print(paste(result2$time["elapsed"]/60, "minutes"))
+print(paste(format(result2$time), "for",length(result2$ess),"SMC iterations."))
 samp.idx <- 1:nrow(result2$location)
 plot(wavenumbers, spectra[1,], type='l', lwd=3,
      xlab=expression(paste("Raman shift (cm"^{-1}, ")")), ylab="Intensity (a.u.)", main="Fitted model with Voigt peaks")
